@@ -1,6 +1,7 @@
 package org.gradle.internal.declarativedsl.analysis
 
 
+import org.gradle.declarative.dsl.evaluation.AnalysisStatementFilter
 import org.gradle.internal.declarativedsl.language.Assignment
 import org.gradle.internal.declarativedsl.language.DataStatement
 import org.gradle.internal.declarativedsl.language.Expr
@@ -21,12 +22,15 @@ class CodeAnalyzerImpl(
         elements: List<DataStatement>
     ) {
         for (element in elements) {
-            if (analysisStatementFilter.shouldAnalyzeStatement(element, context.currentScopes)) {
+            if (analysisStatementFilter.shouldAnalyzeStatement(element, context.isTopLevelScope)) {
                 doResolveStatement(context, element)
             }
         }
     }
 
+    private val AnalysisContextView.isTopLevelScope get() =
+        currentScopes.last().receiver is ObjectOrigin.TopLevelReceiver
+
     private
     fun doResolveStatement(context: AnalysisContext, statement: DataStatement) {
         when (statement) {
