@@ -31,7 +31,6 @@ import org.gradle.api.file.ProjectLayout;
 import org.gradle.api.internal.CollectionCallbackActionDecorator;
 import org.gradle.api.internal.DomainObjectContext;
 import org.gradle.api.internal.GradleInternal;
-import org.gradle.api.internal.artifacts.component.ComponentIdentifierFactory;
 import org.gradle.api.internal.artifacts.configurations.ConfigurationContainerInternal;
 import org.gradle.api.internal.artifacts.configurations.DefaultConfigurationContainer;
 import org.gradle.api.internal.artifacts.configurations.DefaultConfigurationFactory;
@@ -100,7 +99,6 @@ import org.gradle.api.internal.artifacts.transform.TransformRegistrationFactory;
 import org.gradle.api.internal.artifacts.transform.VariantSelectorFactory;
 import org.gradle.api.internal.artifacts.type.ArtifactTypeRegistry;
 import org.gradle.api.internal.artifacts.type.DefaultArtifactTypeRegistry;
-import org.gradle.api.internal.attributes.AttributeDesugaring;
 import org.gradle.api.internal.attributes.AttributesSchemaInternal;
 import org.gradle.api.internal.attributes.DefaultAttributesSchema;
 import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
@@ -113,13 +111,15 @@ import org.gradle.api.internal.project.ProjectStateRegistry;
 import org.gradle.api.internal.provider.PropertyFactory;
 import org.gradle.api.internal.tasks.TaskDependencyFactory;
 import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.problems.internal.InternalProblems;
 import org.gradle.api.provider.ProviderFactory;
 import org.gradle.cache.Cache;
 import org.gradle.cache.ManualEvictionInMemoryCache;
 import org.gradle.internal.authentication.AuthenticationSchemeRegistry;
 import org.gradle.internal.build.BuildModelLifecycleListener;
 import org.gradle.internal.build.BuildState;
-import org.gradle.internal.component.ResolutionFailureHandler;
+import org.gradle.internal.buildoption.InternalOptions;
+import org.gradle.internal.component.resolution.failure.ResolutionFailureHandler;
 import org.gradle.internal.component.external.model.JavaEcosystemVariantDerivationStrategy;
 import org.gradle.internal.component.external.model.ModuleComponentArtifactMetadata;
 import org.gradle.internal.component.model.GraphVariantSelector;
@@ -150,9 +150,11 @@ import org.gradle.internal.resolve.caching.ComponentMetadataRuleExecutor;
 import org.gradle.internal.resource.local.FileResourceListener;
 import org.gradle.internal.resource.local.FileResourceRepository;
 import org.gradle.internal.resource.local.LocallyAvailableResourceFinder;
-import org.gradle.internal.service.DefaultServiceRegistry;
+import org.gradle.internal.service.Provides;
 import org.gradle.internal.service.ServiceRegistration;
+import org.gradle.internal.service.ServiceRegistrationProvider;
 import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.service.ServiceRegistryBuilder;
 import org.gradle.internal.vfs.FileSystemAccess;
 import org.gradle.util.internal.SimpleMapInterner;
 
@@ -171,14 +173,19 @@ public class DefaultDependencyManagementServices implements DependencyManagement
 
     @Override
     public DependencyResolutionServices create(FileResolver resolver, FileCollectionFactory fileCollectionFactory, DependencyMetaDataProvider dependencyMetaDataProvider, ProjectFinder projectFinder, DomainObjectContext domainObjectContext) {
-        DefaultServiceRegistry services = new DefaultServiceRegistry(parent);
-        services.add(FileResolver.class, resolver);
-        services.add(FileCollectionFactory.class, fileCollectionFactory);
-        services.add(DependencyMetaDataProvider.class, dependencyMetaDataProvider);
-        services.add(ProjectFinder.class, projectFinder);
-        services.add(DomainObjectContext.class, domainObjectContext);
-        services.addProvider(new TransformGradleUserHomeServices());
-        services.addProvider(new DependencyResolutionScopeServices(domainObjectContext));
+        ServiceRegistry services = ServiceRegistryBuilder.builder()
+            .parent(parent)
+            .provider(registration -> {
+                registration.add(FileResolver.class, resolver);
+                registration.add(FileCollectionFactory.class, fileCollectionFactory);
+                registration.add(DependencyMetaDataProvider.class, dependencyMetaDataProvider);
+                registration.add(ProjectFinder.class, projectFinder);
+                registration.add(DomainObjectContext.class, domainObjectContext);
+            })
+            .provider(new TransformGradleUserHomeServices())
+            .provider(new DependencyResolutionScopeServices(domainObjectContext))
+            .build();
+
         return services.get(DependencyResolutionServices.class);
     }
 
@@ -187,8 +194,8 @@ public class DefaultDependencyManagementServices implements DependencyManagement
         registration.addProvider(new DependencyResolutionScopeServices(domainObjectContext));
     }
 
-    private static class TransformGradleUserHomeServices {
-
+    private static class TransformGradleUserHomeServices implements ServiceRegistrationProvider {
+        @Provides
         TransformExecutionListener createTransformExecutionListener() {
             return new TransformExecutionListener() {
                 @Override
@@ -202,7 +209,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
         }
     }
 
-    private static class DependencyResolutionScopeServices {
+    private static class DependencyResolutionScopeServices implements ServiceRegistrationProvider {
 
         private final DomainObjectContext domainObjectContext;
 
@@ -213,7 +220,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
         void configure(ServiceRegistration registration) {
             registration.add(DefaultTransformedVariantFactory.class);
             registration.add(DefaultRootComponentMetadataBuilder.Factory.class);
-            registration.add(ResolveExceptionContextualizer.class);
+            registration.add(ResolveExceptionMapper.class);
             registration.add(ResolutionStrategyFactory.class);
             registration.add(DefaultLocalComponentRegistry.class);
             registration.add(ProjectDependencyResolver.class);
@@ -224,6 +231,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             registration.add(DefaultArtifactResolutionQueryFactory.class);
         }
 
+        @Provides
         AttributesSchemaInternal createConfigurationAttributesSchema(InstantiatorFactory instantiatorFactory, IsolatableFactory isolatableFactory, PlatformSupport platformSupport, ServiceRegistry serviceRegistry) {
             DefaultAttributesSchema attributesSchema = instantiatorFactory.decorateLenient().newInstance(DefaultAttributesSchema.class, instantiatorFactory.inject(serviceRegistry), isolatableFactory);
             platformSupport.configureSchema(attributesSchema);
@@ -231,6 +239,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return attributesSchema;
         }
 
+        @Provides
         MutableTransformWorkspaceServices createTransformWorkspaceServices(ProjectLayout projectLayout, ExecutionHistoryStore executionHistoryStore) {
             Supplier<File> baseDirectory = projectLayout.getBuildDirectory().dir(".transforms").map(Directory::getAsFile)::get;
             Cache<UnitOfWork.Identity, IdentityCacheResult<TransformWorkspaceResult>> identityCache = new ManualEvictionInMemoryCache<>();
@@ -252,9 +261,11 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             };
         }
 
+        @Provides
         TransformInvocationFactory createTransformInvocationFactory(
             ExecutionEngine executionEngine,
             FileSystemAccess fileSystemAccess,
+            InternalOptions internalOptions,
             ImmutableTransformWorkspaceServices transformWorkspaceServices,
             TransformExecutionListener transformExecutionListener,
             FileCollectionFactory fileCollectionFactory,
@@ -265,6 +276,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return new DefaultTransformInvocationFactory(
                 executionEngine,
                 fileSystemAccess,
+                internalOptions,
                 transformExecutionListener,
                 transformWorkspaceServices,
                 fileCollectionFactory,
@@ -274,6 +286,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             );
         }
 
+        @Provides
         TransformRegistrationFactory createTransformRegistrationFactory(
             BuildOperationRunner buildOperationRunner,
             IsolatableFactory isolatableFactory,
@@ -304,38 +317,41 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             );
         }
 
+        @Provides
         VariantTransformRegistry createVariantTransformRegistry(InstantiatorFactory instantiatorFactory, ImmutableAttributesFactory attributesFactory, ServiceRegistry services, TransformRegistrationFactory transformRegistrationFactory, TransformParameterScheme parameterScheme) {
             return new DefaultVariantTransformRegistry(instantiatorFactory, attributesFactory, services, transformRegistrationFactory, parameterScheme.getInstantiationScheme());
         }
 
+        @Provides
         DefaultUrlArtifactRepository.Factory createDefaultUrlArtifactRepositoryFactory(FileResolver fileResolver) {
             return new DefaultUrlArtifactRepository.Factory(fileResolver);
         }
 
+        @Provides
         BaseRepositoryFactory createBaseRepositoryFactory(
-                LocalMavenRepositoryLocator localMavenRepositoryLocator,
-                FileResolver fileResolver,
-                FileCollectionFactory fileCollectionFactory,
-                RepositoryTransportFactory repositoryTransportFactory,
-                LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> locallyAvailableResourceFinder,
-                FileStoreAndIndexProvider fileStoreAndIndexProvider,
-                VersionSelectorScheme versionSelectorScheme,
-                AuthenticationSchemeRegistry authenticationSchemeRegistry,
-                IvyContextManager ivyContextManager,
-                ImmutableAttributesFactory attributesFactory,
-                ImmutableModuleIdentifierFactory moduleIdentifierFactory,
-                InstantiatorFactory instantiatorFactory,
-                FileResourceRepository fileResourceRepository,
-                MavenMutableModuleMetadataFactory metadataFactory,
-                IvyMutableModuleMetadataFactory ivyMetadataFactory,
-                IsolatableFactory isolatableFactory,
-                ObjectFactory objectFactory,
-                CollectionCallbackActionDecorator callbackDecorator,
-                NamedObjectInstantiator instantiator,
-                DefaultUrlArtifactRepository.Factory urlArtifactRepositoryFactory,
-                ChecksumService checksumService,
-                ProviderFactory providerFactory,
-                VersionParser versionParser
+            LocalMavenRepositoryLocator localMavenRepositoryLocator,
+            FileResolver fileResolver,
+            FileCollectionFactory fileCollectionFactory,
+            RepositoryTransportFactory repositoryTransportFactory,
+            LocallyAvailableResourceFinder<ModuleComponentArtifactMetadata> locallyAvailableResourceFinder,
+            FileStoreAndIndexProvider fileStoreAndIndexProvider,
+            VersionSelectorScheme versionSelectorScheme,
+            AuthenticationSchemeRegistry authenticationSchemeRegistry,
+            IvyContextManager ivyContextManager,
+            ImmutableAttributesFactory attributesFactory,
+            ImmutableModuleIdentifierFactory moduleIdentifierFactory,
+            InstantiatorFactory instantiatorFactory,
+            FileResourceRepository fileResourceRepository,
+            MavenMutableModuleMetadataFactory metadataFactory,
+            IvyMutableModuleMetadataFactory ivyMetadataFactory,
+            IsolatableFactory isolatableFactory,
+            ObjectFactory objectFactory,
+            CollectionCallbackActionDecorator callbackDecorator,
+            NamedObjectInstantiator instantiator,
+            DefaultUrlArtifactRepository.Factory urlArtifactRepositoryFactory,
+            ChecksumService checksumService,
+            ProviderFactory providerFactory,
+            VersionParser versionParser
         ) {
             return new DefaultBaseRepositoryFactory(
                 localMavenRepositoryLocator,
@@ -364,13 +380,18 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             );
         }
 
+        @Provides
         RepositoryHandler createRepositoryHandler(Instantiator instantiator, BaseRepositoryFactory baseRepositoryFactory, CollectionCallbackActionDecorator callbackDecorator) {
             return instantiator.newInstance(DefaultRepositoryHandler.class, baseRepositoryFactory, instantiator, callbackDecorator);
         }
 
+        @Provides
         ConfigurationContainerInternal createConfigurationContainer(
             Instantiator instantiator,
             CollectionCallbackActionDecorator callbackDecorator,
+            DependencyMetaDataProvider dependencyMetaDataProvider,
+            DomainObjectContext domainObjectContext,
+            AttributesSchemaInternal attributesSchema,
             DefaultRootComponentMetadataBuilder.Factory rootComponentMetadataBuilderFactory,
             DefaultConfigurationFactory defaultConfigurationFactory,
             ResolutionStrategyFactory resolutionStrategyFactory
@@ -378,12 +399,16 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return instantiator.newInstance(DefaultConfigurationContainer.class,
                 instantiator,
                 callbackDecorator,
+                dependencyMetaDataProvider,
+                domainObjectContext,
+                attributesSchema,
                 rootComponentMetadataBuilderFactory,
                 defaultConfigurationFactory,
                 resolutionStrategyFactory
             );
         }
 
+        @Provides
         PublishArtifactNotationParserFactory createPublishArtifactNotationParserFactory(
             Instantiator instantiator,
             DependencyMetaDataProvider metaDataProvider,
@@ -398,23 +423,27 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             );
         }
 
+        @Provides
         ArtifactTypeRegistry createArtifactTypeRegistry(Instantiator instantiator, ImmutableAttributesFactory immutableAttributesFactory, CollectionCallbackActionDecorator decorator, VariantTransformRegistry transformRegistry) {
             return new DefaultArtifactTypeRegistry(instantiator, immutableAttributesFactory, decorator, transformRegistry);
         }
 
-        DependencyHandler createDependencyHandler(Instantiator instantiator,
-                                                  ConfigurationContainerInternal configurationContainer,
-                                                  DependencyFactoryInternal dependencyFactory,
-                                                  ProjectFinder projectFinder,
-                                                  DependencyConstraintHandler dependencyConstraintHandler,
-                                                  ComponentMetadataHandler componentMetadataHandler,
-                                                  ComponentModuleMetadataHandler componentModuleMetadataHandler,
-                                                  ArtifactResolutionQueryFactory resolutionQueryFactory,
-                                                  AttributesSchema attributesSchema,
-                                                  VariantTransformRegistry variantTransformRegistry,
-                                                  ArtifactTypeRegistry artifactTypeRegistry,
-                                                  ObjectFactory objects,
-                                                  PlatformSupport platformSupport) {
+        @Provides
+        DependencyHandler createDependencyHandler(
+            Instantiator instantiator,
+            ConfigurationContainerInternal configurationContainer,
+            DependencyFactoryInternal dependencyFactory,
+            ProjectFinder projectFinder,
+            DependencyConstraintHandler dependencyConstraintHandler,
+            ComponentMetadataHandler componentMetadataHandler,
+            ComponentModuleMetadataHandler componentModuleMetadataHandler,
+            ArtifactResolutionQueryFactory resolutionQueryFactory,
+            AttributesSchema attributesSchema,
+            VariantTransformRegistry variantTransformRegistry,
+            ArtifactTypeRegistry artifactTypeRegistry,
+            ObjectFactory objects,
+            PlatformSupport platformSupport
+        ) {
             return instantiator.newInstance(DefaultDependencyHandler.class,
                 configurationContainer,
                 dependencyFactory,
@@ -430,6 +459,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
                 platformSupport);
         }
 
+        @Provides
         DependencyLockingHandler createDependencyLockingHandler(Instantiator instantiator, ConfigurationContainerInternal configurationContainer, DependencyLockingProvider dependencyLockingProvider) {
             if (domainObjectContext.isPluginContext()) {
                 throw new IllegalStateException("Cannot use locking handler in plugins context");
@@ -438,6 +468,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return instantiator.newInstance(DefaultDependencyLockingHandler.class, (Supplier<ConfigurationContainerInternal>) () -> configurationContainer, dependencyLockingProvider);
         }
 
+        @Provides
         DependencyLockingProvider createDependencyLockingProvider(FileResolver fileResolver, StartParameter startParameter, DomainObjectContext context, GlobalDependencyResolutionRules globalDependencyResolutionRules, ListenerManager listenerManager, PropertyFactory propertyFactory, FilePropertyFactory filePropertyFactory) {
             if (domainObjectContext.isPluginContext()) {
                 return NoOpDependencyLockingProvider.getInstance();
@@ -457,17 +488,21 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return dependencyLockingProvider;
         }
 
+        @Provides
         DependencyConstraintHandler createDependencyConstraintHandler(Instantiator instantiator, ConfigurationContainerInternal configurationContainer, DependencyConstraintFactoryInternal dependencyConstraintFactory, ObjectFactory objects, PlatformSupport platformSupport) {
             return instantiator.newInstance(DefaultDependencyConstraintHandler.class, configurationContainer, dependencyConstraintFactory, objects, platformSupport);
         }
 
-        DefaultComponentMetadataHandler createComponentMetadataHandler(Instantiator instantiator,
-                                                                       ImmutableModuleIdentifierFactory moduleIdentifierFactory,
-                                                                       SimpleMapInterner interner,
-                                                                       ImmutableAttributesFactory attributesFactory,
-                                                                       IsolatableFactory isolatableFactory,
-                                                                       ComponentMetadataRuleExecutor componentMetadataRuleExecutor,
-                                                                       PlatformSupport platformSupport) {
+        @Provides
+        DefaultComponentMetadataHandler createComponentMetadataHandler(
+            Instantiator instantiator,
+            ImmutableModuleIdentifierFactory moduleIdentifierFactory,
+            SimpleMapInterner interner,
+            ImmutableAttributesFactory attributesFactory,
+            IsolatableFactory isolatableFactory,
+            ComponentMetadataRuleExecutor componentMetadataRuleExecutor,
+            PlatformSupport platformSupport
+        ) {
             DefaultComponentMetadataHandler componentMetadataHandler = instantiator.newInstance(DefaultComponentMetadataHandler.class, instantiator, moduleIdentifierFactory, interner, attributesFactory, isolatableFactory, componentMetadataRuleExecutor, platformSupport);
             if (domainObjectContext.isScript()) {
                 componentMetadataHandler.setVariantDerivationStrategy(JavaEcosystemVariantDerivationStrategy.getInstance());
@@ -475,14 +510,17 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return componentMetadataHandler;
         }
 
+        @Provides
         DefaultComponentModuleMetadataHandler createComponentModuleMetadataHandler(Instantiator instantiator, ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
             return instantiator.newInstance(DefaultComponentModuleMetadataHandler.class, moduleIdentifierFactory);
         }
 
+        @Provides
         ArtifactHandler createArtifactHandler(Instantiator instantiator, ConfigurationContainerInternal configurationContainer, PublishArtifactNotationParserFactory publishArtifactNotationParserFactory) {
             return instantiator.newInstance(DefaultArtifactHandler.class, configurationContainer, publishArtifactNotationParserFactory.create());
         }
 
+        @Provides
         ComponentMetadataProcessorFactory createComponentMetadataProcessorFactory(ComponentMetadataHandlerInternal componentMetadataHandler, DependencyResolutionManagementInternal dependencyResolutionManagement, DomainObjectContext context) {
             if (context.isScript()) {
                 return componentMetadataHandler::createComponentMetadataProcessor;
@@ -490,25 +528,28 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             return componentMetadataHandler.createFactory(dependencyResolutionManagement);
         }
 
+        @Provides
         GlobalDependencyResolutionRules createModuleMetadataHandler(ComponentMetadataProcessorFactory componentMetadataProcessorFactory, ComponentModuleMetadataProcessor moduleMetadataProcessor, List<DependencySubstitutionRules> rules) {
             return new DefaultGlobalDependencyResolutionRules(componentMetadataProcessorFactory, moduleMetadataProcessor, rules);
         }
 
-        ResolutionFailureHandler createResolutionFailureProcessor(InstantiatorFactory instantiatorFactory, ServiceRegistry serviceRegistry) {
+        @Provides
+        ResolutionFailureHandler createResolutionFailureProcessor(InstantiatorFactory instantiatorFactory, ServiceRegistry serviceRegistry, InternalProblems problemsService) {
             InstanceGenerator instanceGenerator = instantiatorFactory.inject(serviceRegistry);
             ResolutionFailureDescriberRegistry failureDescriberRegistry = ResolutionFailureDescriberRegistry.standardRegistry(instanceGenerator);
-            return new ResolutionFailureHandler(failureDescriberRegistry);
+            return new ResolutionFailureHandler(failureDescriberRegistry, problemsService);
         }
 
+        @Provides
         GraphVariantSelector createGraphVariantSelector(ResolutionFailureHandler resolutionFailureHandler) {
             return new GraphVariantSelector(resolutionFailureHandler);
         }
 
+        @Provides
         ConfigurationResolver createDependencyResolver(
             DependencyGraphResolver dependencyGraphResolver,
             RepositoriesSupplier repositoriesSupplier,
             GlobalDependencyResolutionRules metadataHandler,
-            ComponentIdentifierFactory componentIdentifierFactory,
             ResolutionResultsStoreFactory resolutionResultsStoreFactory,
             StartParameter startParameter,
             AttributesSchemaInternal attributesSchema,
@@ -522,8 +563,6 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             BuildState currentBuild,
             ComponentSelectionDescriptorFactory componentSelectionDescriptorFactory,
             ResolvedArtifactSetResolver artifactSetResolver,
-            AttributeDesugaring attributeDesugaring,
-            ResolveExceptionContextualizer resolveExceptionContextualizer,
             ComponentDetailsSerializer componentDetailsSerializer,
             SelectedVariantSerializer selectedVariantSerializer,
             ResolvedVariantCache resolvedVariantCache,
@@ -550,10 +589,8 @@ public class DefaultDependencyManagementServices implements DependencyManagement
                 componentSelectorConverter,
                 attributeContainerSerializer,
                 currentBuild,
-                attributeDesugaring,
                 artifactSetResolver,
                 componentSelectionDescriptorFactory,
-                resolveExceptionContextualizer,
                 componentDetailsSerializer,
                 selectedVariantSerializer,
                 resolvedVariantCache,
@@ -566,22 +603,20 @@ public class DefaultDependencyManagementServices implements DependencyManagement
                 dependencyLockingProvider
             );
 
-            return new ShortCircuitEmptyConfigurationResolver(
-                defaultResolver,
-                componentIdentifierFactory,
-                moduleIdentifierFactory,
-                currentBuild.getBuildIdentifier()
-            );
+            return new ShortCircuitEmptyConfigurationResolver(defaultResolver);
         }
 
+        @Provides
         ArtifactPublicationServices createArtifactPublicationServices(ServiceRegistry services) {
             return new DefaultArtifactPublicationServices(services);
         }
 
+        @Provides
         DependencyResolutionServices createDependencyResolutionServices(ServiceRegistry services) {
             return new DefaultDependencyResolutionServices(services);
         }
 
+        @Provides
         RepositoriesSupplier createRepositoriesSupplier(RepositoryHandler repositoryHandler, DependencyResolutionManagementInternal drm, DomainObjectContext context) {
             return () -> {
                 List<ResolutionAwareRepository> repositories = collectRepositories(repositoryHandler);
